@@ -15,7 +15,7 @@ use Module::Build::Base;
 
 use vars qw($VERSION @ISA);
 @ISA = qw(Module::Build::Base);
-$VERSION = '0.27_01';
+$VERSION = '0.27_02';
 $VERSION = eval $VERSION;
 
 # Okay, this is the brute-force method of finding out what kind of
@@ -154,14 +154,14 @@ action), 'test', and 'install'.  Other actions defined so far include:
   config_data                    manifest    
   diff                           ppd         
   dist                           ppmdist     
-  distcheck                      realclean   
-  distclean                      skipcheck   
-  distdir                        test        
-  distmeta                       testcover   
-  distsign                       testdb      
-  disttest                       testpod     
-  docs                           versioninstall
-  fakeinstall                                
+  distcheck                      pure_install
+  distclean                      realclean   
+  distdir                        skipcheck   
+  distmeta                       test        
+  distsign                       testcover   
+  disttest                       testdb      
+  docs                           testpod     
+  fakeinstall                    versioninstall
 
 
 You can run the 'help' action for a complete list of actions.
@@ -407,6 +407,13 @@ This uses the same mechanism as the C<dist> action to tar & zip its
 output, so you can supply C<tar> and/or C<gzip> parameters to affect
 the result.
 
+=item pure_install
+
+This action is identical to the C<install> action.  In the future,
+though, if C<install> starts writing to the file file
+F<$(INSTALLARCHLIB)/perllocal.pod>, C<pure_install> won't, and that
+will be the only difference between them.
+
 =item realclean
 
 This action is just like the C<clean> action, but also removes the
@@ -748,11 +755,17 @@ in Module::Build, here's a summary of the reasons why:
 Many systems have Perl configs that make little sense with PREFIX.
 For example, OS X, where core modules go in
 F</System/Library/Perl/...>, user-installed modules go in
-F</Library/Perl/...>, and man pages go in F</usr/share/man/...>.
+F</Library/Perl/...>, and man pages go in F</usr/share/man/...>.  The PREFIX is thus set to F</>.
 Install L<Foo::Bar> on OS X with C<PREFIX=/home/spurkis> and you get
 things like F</home/spurkis/Library/Perl/5.8.1/Foo/Bar.pm> and
 F</home/spurkis/usr/share/man/man3/Foo::Bar.3pm>.  Not too pretty.
 
+The problem is not limited to Unix-like platforms, either - on Windows
+builds (e.g. ActiveState perl 5.8.0), we have user-installed modules
+going in F<C:\Perl\site\lib>, user-installed executables going in
+F<C:\Perl\bin>, and PREFIX=F<C:\Perl\site>.  The prefix just doesn't
+apply neatly to the executables.
+
 =item *
 
 The PREFIX logic is too complicated and hard to predict for the user.
