@@ -81,8 +81,15 @@ sub new_from_context {
   
   # XXX Read the META.yml and see whether we need to run the Build.PL?
   
-  # Run the Build.PL
-  $package->run_perl_script('Build.PL', [], [$package->unparse_args(\%args)]);
+  # Run the Build.PL.  We use do() rather than run_perl_script() so
+  # that it runs in this process rather than a subprocess, because we
+  # need to make sure that the environment is the same during Build.PL
+  # as it is during resume() (and thereafter).
+  {
+    local @ARGV = $package->unparse_args(\%args);
+    do 'Build.PL';
+    die $@ if $@;
+  }
   return $package->resume;
 }
 
@@ -124,7 +131,6 @@ sub _construct {
     }
   }
 
-
   # The following warning could be unnecessary if the user is running
   # an embedded perl, but there aren't too many of those around, and
   # embedded perls aren't usually used to install modules, and the
@@ -133,8 +139,9 @@ sub _construct {
   $p->{perl} = $self->find_perl_interpreter
     or $self->log_warn("Warning: Can't locate your perl binary");
 
-  $p->{bindoc_dirs} ||= [ "$p->{blib}/script" ];
-  $p->{libdoc_dirs} ||= [ "$p->{blib}/lib", "$p->{blib}/arch" ];
+  my $blibdir = sub { File::Spec->catdir($p->{blib}, @_) };
+  $p->{bindoc_dirs} ||= [ $blibdir->("script") ];
+  $p->{libdoc_dirs} ||= [ $blibdir->("lib"), $blibdir->("arch") ];
 
   $p->{dist_author} = [ $p->{dist_author} ] if defined $p->{dist_author} and not ref $p->{dist_author};
 
@@ -166,10 +173,14 @@ sub log_warn {
 sub _set_install_paths {
   my $self = shift;
   my $c = $self->{config};
+  my $p = $self->{properties};
 
-  my @html = $c->{installhtmldir} ? (html => $c->{installhtmldir}) : ();
+  my @libstyle = $c->{installstyle} ? File::Spec->splitdir($c->{installstyle}) : qw(lib perl5);
+  my $arch     = $c->{archname};
+  my $version  = $c->{version};
+  my @html     = $c->{installhtmldir} ? (html => $c->{installhtmldir}) : ();
 
-  $self->{properties}{install_sets} =
+  $p->{install_sets} =
     {
      core   => {
 		lib     => $c->{installprivlib},
@@ -199,6 +210,53 @@ sub _set_install_paths {
 		@html,
 	       },
     };
+
+  $p->{original_prefix} =
+    {
+     core   => $c->{installprefixexp} || $c->{installprefix} ||
+               $c->{prefixexp}        || $c->{prefix} || '',
+     site   => $c->{siteprefixexp},
+     vendor => $c->{usevendorprefix} ? $c->{vendorprefixexp} : '',
+    };
+  $p->{original_prefix}{site} ||= $p->{original_prefix}{core};
+
+  $p->{install_base_relpaths} =
+    {
+     lib     => ['lib', 'perl5'],
+     arch    => ['lib', 'perl5', $arch],
+     bin     => ['bin'],
+     script  => ['bin'],
+     bindoc  => ['man', 'man1'],
+     libdoc  => ['man', 'man3'],
+    };
+
+  $p->{prefix_relpaths} = 
+    {
+     core => {
+	      lib        => [@libstyle],
+	      arch       => [@libstyle, $version, $arch],
+	      bin        => ['bin'],
+	      script     => ['bin'],
+	      libdoc     => ['man', 'man3'],
+	      bindoc     => ['man', 'man1'],
+	     },
+     vendor => {
+		lib        => [@libstyle],
+		arch       => [@libstyle, $version, $arch],
+		bin        => ['bin'],
+		script     => ['bin'],
+		libdoc     => ['man', 'man3'],
+		bindoc     => ['man', 'man1'],
+	       },
+     site => {
+	      lib        => [@libstyle, 'site_perl'],
+	      arch       => [@libstyle, 'site_perl', $version, $arch],
+	      bin        => ['bin'],
+	      script     => ['bin'],
+	      libdoc     => ['man', 'man3'],
+	      bindoc     => ['man', 'man1'],
+	     },
+    };
 }
 
 sub _find_nested_builds {
@@ -253,7 +311,6 @@ sub find_perl_interpreter {
 }
 
 sub base_dir { shift()->{properties}{base_dir} }
-sub installdirs { shift()->{properties}{installdirs} }
 
 sub _is_interactive {
   return -t STDIN && (-t STDOUT || !(-f STDOUT || -c STDOUT)) ;   # Pipe?
@@ -492,6 +549,9 @@ __PACKAGE__->add_property($_) for qw(
    perl
    has_config_data
    install_sets
+   install_base_relpaths
+   original_prefix
+   prefix_relpaths
    install_base
    destdir
    debugger
@@ -510,6 +570,7 @@ __PACKAGE__->add_property($_) for qw(
    ignore_prereq_requires
    ignore_prereqs
    skip_rcfile
+   prefix
 );
 
 {
@@ -589,8 +650,8 @@ sub subclass {
   my $fh = IO::File->new("> $filename") or die "Can't create $filename: $!";
   print $fh <<EOF;
 package $opts{class};
-use Module::Build;
-\@ISA = qw(Module::Build);
+use $pack;
+\@ISA = qw($pack);
 $opts{code}
 1;
 EOF
@@ -619,21 +680,24 @@ sub dist_name {
 sub dist_version {
   my ($self) = @_;
   my $p = $self->{properties};
-  
+
   return $p->{dist_version} if defined $p->{dist_version};
-  
+
   if ($self->module_name) {
     $p->{dist_version_from} ||= join( '/', 'lib', split '::', $self->module_name ) . '.pm';
   }
-  
+
+  if ( $p->{dist_version_from} ) {
+    my $version_from = File::Spec->catfile( split( qr{/}, $p->{dist_version_from} ) );
+    my $pm_info = Module::Build::ModuleInfo->new_from_file( $version_from );
+    $p->{dist_version} = $pm_info->version();
+  }
+
   die ("Can't determine distribution version, must supply either 'dist_version',\n".
        "'dist_version_from', or 'module_name' parameter")
-    unless $p->{dist_version_from};
-  
-  my $version_from = File::Spec->catfile( split '/', $p->{dist_version_from} );
-  
-  my $pm_info = Module::Build::ModuleInfo->new_from_file( $version_from );
-  return $p->{dist_version} = $pm_info->version();
+    unless $p->{dist_version};
+
+  return $p->{dist_version};
 }
 
 sub dist_author   { shift->_pod_parse('author')   }
@@ -923,17 +987,21 @@ sub _added_to_INC {
 }
 
 # Determine the default @INC for this Perl
-sub _default_INC {
-  my $self = shift;
-
-  local $ENV{PERL5LIB};  # this is not considered part of the default.
-
-  my $perl = ref($self) ? $self->perl : $self->find_perl_interpreter;
-
-  my @inc = `$perl -le "print for \@INC"`;
-  chomp @inc;
-
-  return @inc;
+{
+  my @default_inc; # Memoize
+  sub _default_INC {
+    my $self = shift;
+    return @default_inc if @default_inc;
+    
+    local $ENV{PERL5LIB};  # this is not considered part of the default.
+    
+    my $perl = ref($self) ? $self->perl : $self->find_perl_interpreter;
+    
+    my @inc = `$perl -le "print for \@INC"`;
+    chomp @inc;
+    
+    return @default_inc = @inc;
+  }
 }
 
 sub print_build_script {
@@ -1200,7 +1268,22 @@ sub read_args {
     }
     $args{$_} = \%hash;
   }
-  
+
+  # De-tilde-ify any path parameters
+  for my $key (qw(prefix install_base destdir)) {
+    next if !defined $args{$key};
+    $args{$key} = _detildefy($args{$key});
+  }
+
+  for my $key (qw(install_path)) {
+    next if !defined $args{$key};
+
+    for my $subkey (keys %{$args{$key}}) {
+      next if !defined $args{$key}{$subkey};
+      $args{$key}{$subkey} = _detildefy($args{$key}{$subkey});
+    }
+  }
+
   if ($args{makefile_env_macros}) {
     require Module::Build::Compat;
     %args = (%args, Module::Build::Compat->makefile_to_build_macros);
@@ -1209,6 +1292,16 @@ sub read_args {
   return \%args, $action;
 }
 
+
+sub _detildefy {
+    my $arg = shift;
+
+    ($arg) = $arg =~ /^~/ ? glob($arg) : $arg;
+
+    return $arg;
+}
+
+
 # merge Module::Build argument lists that have already been parsed
 # by read_args(). Takes two references to option hashes and merges
 # the contents, giving priority to the first.
@@ -1231,14 +1324,26 @@ sub _merge_arglist {
   return %new_opts;
 }
 
+# Look for a home directory on various systems.  CPANPLUS does something like this.
+sub _home_dir {
+  my @os_home_envs = qw( APPDATA HOME USERPROFILE WINDIR SYS$LOGIN );
+  
+  foreach ( @os_home_envs ) {
+    return $ENV{$_} if exists $ENV{$_} && defined $ENV{$_} && length $ENV{$_} && -d $ENV{$_};
+  }
+  
+  return;
+}
+
 # read ~/.modulebuildrc returning global options '*' and
 # options specific to the currently executing $action.
 sub read_modulebuildrc {
   my( $self, $action ) = @_;
 
-  return () unless exists( $ENV{HOME} ) && -e $ENV{HOME};
+  my $home = $self->_home_dir;
+  return () unless defined $home;
 
-  my $modulebuildrc = File::Spec->catfile( $ENV{HOME}, '.modulebuildrc' );
+  my $modulebuildrc = File::Spec->catfile( $home, '.modulebuildrc' );
   return () unless -e $modulebuildrc;
 
   my $fh = IO::File->new( $modulebuildrc )
@@ -1692,10 +1797,14 @@ sub _find_file_by_type {
 
 sub localize_file_path {
   my ($self, $path) = @_;
-  return $path unless $path =~ m{/};
   return File::Spec->catfile( split m{/}, $path );
 }
 
+sub localize_dir_path {
+  my ($self, $path) = @_;
+  return File::Spec->catdir( split m{/}, $path );
+}
+
 sub fix_shebang_line { # Adapted from fixin() in ExtUtils::MM_Unix 1.35
   my ($self, @files) = @_;
   my $c = $self->{config};
@@ -1827,7 +1936,7 @@ sub _find_pods {
   my ($self, $dirs, %args) = @_;
   my %files;
   foreach my $spec (@$dirs) {
-    my $dir = $self->localize_file_path($spec);
+    my $dir = $self->localize_dir_path($spec);
     next unless -e $dir;
     FILE: foreach my $file ( @{ $self->rscan_dir( $dir ) } ) {
       foreach my $regexp ( @{ $args{exclude} } ) {
@@ -1873,7 +1982,7 @@ sub htmlify_pods {
 	        map  [File::Spec->catdir($self->blib, $_), $_],
 		qw( script lib );
 
-  my $htmldir = File::Spec::Unix->catdir($self->blib, 'html');
+  my $htmldir = File::Spec->catdir($self->blib, 'html');
   unless (-d $htmldir) {
     File::Path::mkpath($htmldir, 0, 0755) or die "Couldn't mkdir $htmldir: $!";
   }
@@ -1895,9 +2004,9 @@ sub htmlify_pods {
     my @dirs = File::Spec->splitdir( File::Spec->canonpath( $path ) );
     pop( @dirs ) if $dirs[-1] eq File::Spec->curdir;
 
-    my $fulldir = File::Spec::Unix->catfile($htmldir, @rootdirs, @dirs);
-    my $outfile = File::Spec::Unix->catfile($fulldir, $name . '.html');
-    my $infile  = File::Spec::Unix->abs2rel($pod);
+    my $fulldir = File::Spec->catfile($htmldir, @rootdirs, @dirs);
+    my $outfile = File::Spec->catfile($fulldir, $name . '.html');
+    my $infile  = File::Spec->abs2rel($pod);
 
     return if $self->up_to_date($infile, $outfile);
 
@@ -1907,7 +2016,7 @@ sub htmlify_pods {
     }
 
     my $path2root = "../" x (@rootdirs+@dirs);
-    my $htmlroot = File::Spec::Unix->catdir($path2root, 'site');
+    my $htmlroot = "$path2root/site";
 
     my $title = join('::', (@dirs, $name));
     {
@@ -2010,6 +2119,10 @@ sub ACTION_diff {
   }
 }
 
+sub ACTION_pure_install {
+  shift()->depends_on('install');
+}
+
 sub ACTION_install {
   my ($self) = @_;
   require ExtUtils::Install;
@@ -2165,14 +2278,19 @@ sub ACTION_distclean {
 sub do_create_makefile_pl {
   my $self = shift;
   require Module::Build::Compat;
+  $self->delete_filetree('Makefile.PL');
+  $self->log_info("Creating Makefile.PL\n");
   Module::Build::Compat->create_makefile_pl($self->create_makefile_pl, $self, @_);
   $self->_add_to_manifest('MANIFEST', 'Makefile.PL');
 }
 
 sub do_create_readme {
   my $self = shift;
-  require Pod::Text;
-  my $parser = Pod::Text->new;
+  $self->delete_filetree('README');
+  my $parser = eval {require Pod::Readme; 1} ? Pod::Readme->new :
+               eval {require Pod::Text;   1} ? Pod::Text->new :
+	       die "Can't load Pod::Readme or Pod::Text to create README";
+  $self->log_info("Creating README using " . ref($parser) . "\n");
   $parser->parse_from_file($self->dist_version_from, 'README', @_);
   $self->_add_to_manifest('MANIFEST', 'README');
 }
@@ -2182,9 +2300,6 @@ sub ACTION_distdir {
 
   $self->depends_on('distmeta');
 
-  $self->do_create_makefile_pl if $self->create_makefile_pl;
-  $self->do_create_readme if $self->create_readme;
-  
   my $dist_files = $self->_read_manifest('MANIFEST')
     or die "Can't create distdir without a MANIFEST file - run 'manifest' action first";
   delete $dist_files->{SIGNATURE};  # Don't copy, create a fresh one
@@ -2196,6 +2311,7 @@ sub ACTION_distdir {
   
   my $dist_dir = $self->dist_dir;
   $self->delete_filetree($dist_dir);
+  $self->log_info("Creating $dist_dir\n");
   $self->add_to_cleanup($dist_dir);
   
   foreach my $file (keys %$dist_files) {
@@ -2380,6 +2496,14 @@ END_OF_META
 
 sub ACTION_distmeta {
   my ($self) = @_;
+
+  $self->do_create_makefile_pl if $self->create_makefile_pl;
+  $self->do_create_readme if $self->create_readme;
+  $self->do_create_meta_yml;
+}
+
+sub do_create_meta_yml {
+  my $self = shift;
   return if $self->{wrote_metadata};
   
   my $p = $self->{properties};
@@ -2395,6 +2519,7 @@ sub ACTION_distmeta {
 
   # If we're in the distdir, the metafile may exist and be non-writable.
   $self->delete_filetree($self->{metafile});
+  $self->log_info("Creating $self->{metafile}\n");
 
   # Since we're building ourself, we have to do some special stuff
   # here: the ConfigData module is found in blib/lib.
@@ -2467,7 +2592,7 @@ sub _read_manifest {
 
 sub find_dist_packages {
   my $self = shift;
-  
+
   # Only packages in .pm files are candidates for inclusion here.
   # Only include things in the MANIFEST, not things in developer's
   # private stock.
@@ -2480,21 +2605,145 @@ sub find_dist_packages {
                        keys %$manifest;
 
   my @pm_files = grep {exists $dist_files{$_}} keys %{ $self->find_pm_files };
-  
-  my %out;
-  foreach my $file (@pm_files) {
-    next if $file =~ m{^t/};  # Skip things in t/
-    
-    my $localfile = File::Spec->catfile( split m{/}, $file );
 
-    my $pm_info = Module::Build::ModuleInfo->new_from_file( $localfile );
-    
-    foreach my $package ($pm_info->packages_inside($localfile)) {
-      $out{$package}{file} = $dist_files{$file};
-      $out{$package}{version} = $pm_info->version( $package );
+  # First, we enumerate all packages & versions,
+  # seperating into primary & alternative candidates
+  my( %prime, %alt );
+  foreach my $file (@pm_files) {
+    next if $dist_files{$file} =~ m{^t/};  # Skip things in t/
+
+    my @path = split( /\//, $dist_files{$file} );
+    (my $prime_package = join( '::', @path[1..$#path] )) =~ s/\.pm$//;
+
+    my $pm_info = Module::Build::ModuleInfo->new_from_file( $file );
+
+    foreach my $package ( $pm_info->packages_inside ) {
+      next if $package eq 'main';  # main can appear numerous times, ignore
+      next if grep /^_/, split( /::/, $package ); # private package, ignore
+
+      my $version = $pm_info->version( $package );
+
+      if ( $package eq $prime_package ) {
+	if ( exists( $prime{$package} ) ) {
+	  # M::B::ModuleInfo will handle this conflict
+	  die "Unexpected conflict in '$package'; multiple versions found.\n";
+	} else {
+	  $prime{$package}{file} = $dist_files{$file};
+          $prime{$package}{version} = $version if defined( $version );
+        }
+      } else {
+	push( @{$alt{$package}}, {
+				  file    => $dist_files{$file},
+				  version => $version,
+			         } );
+      }
     }
   }
-  return \%out;
+
+  # Then we iterate over all the packages found above, identifying conflicts
+  # and selecting the "best" candidate for recording the file & version
+  # for each package.
+  foreach my $package ( keys( %alt ) ) {
+    my $result = $self->_resolve_module_versions( $alt{$package} );
+
+    if ( exists( $prime{$package} ) ) { # primary package selected
+
+      if ( $result->{err} ) {
+	# Use the selected primary package, but there are conflicting
+	# errors amoung multiple alternative packages that need to be
+	# reported
+        $self->log_warn(
+	  "Found conflicting versions for package '$package'\n" .
+	  "  $prime{$package}{file} ($prime{$package}{version})\n" .
+	  $result->{err}
+        );
+
+      } elsif ( defined( $result->{version} ) ) {
+	# There is a primary package selected, and exactly one
+	# alternative package
+
+	if ( exists( $prime{$package}{version} ) &&
+	     defined( $prime{$package}{version} ) ) {
+	  # Unless the version of the primary package agrees with the
+	  # version of the alternative package, report a conflict
+	  if ( $self->compare_versions( $prime{$package}{version}, '!=',
+					$result->{version} ) ) {
+            $self->log_warn(
+              "Found conflicting versions for package '$package'\n" .
+	      "  $prime{$package}{file} ($prime{$package}{version})\n" .
+	      "  $result->{file} ($result->{version})\n"
+            );
+	  }
+
+	} else {
+	  # The prime package selected has no version so, we choose to
+	  # use any alternative package that does have a version
+	  $prime{$package}{file}    = $result->{file};
+	  $prime{$package}{version} = $result->{version};
+	}
+
+      } else {
+	# no alt package found with a version, but we have a prime
+	# package so we use it whether it has a version or not
+      }
+
+    } else { # No primary package was selected, use the best alternative
+
+      if ( $result->{err} ) {
+        $self->log_warn(
+          "Found conflicting versions for package '$package'\n" .
+	  $result->{err}
+        );
+      }
+
+      # Despite possible conflicting versions, we choose to record
+      # something rather than nothing
+      $prime{$package}{file}    = $result->{file};
+      $prime{$package}{version} = $result->{version}
+	  if defined( $result->{version} );
+    }
+  }
+
+  return \%prime;
+}
+
+# seperate out some of the conflict resolution logic from
+# $self->find_dist_packages(), above, into a helper function.
+#
+sub _resolve_module_versions {
+  my $self = shift;
+
+  my $packages = shift;
+
+  my( $file, $version );
+  my $err = '';
+    foreach my $p ( @$packages ) {
+      if ( defined( $p->{version} ) ) {
+	if ( defined( $version ) ) {
+ 	  if ( $self->compare_versions( $version, '!=', $p->{version} ) ) {
+	    $err .= "  $p->{file} ($p->{version})\n";
+	  } else {
+	    # same version declared multiple times, ignore
+	  }
+	} else {
+	  $file    = $p->{file};
+	  $version = $p->{version};
+	}
+      }
+      $file ||= $p->{file} if defined( $p->{file} );
+    }
+
+  if ( $err ) {
+    $err = "  $file ($version)\n" . $err;
+  }
+
+  my %result = (
+    file    => $file,
+    version => $version,
+    err     => $err
+  );
+
+  return \%result;
 }
 
 sub make_tarball {
@@ -2517,27 +2766,85 @@ sub make_tarball {
   }
 }
 
-sub install_base_relative {
-  my ($self, $type) = @_;
-  # XXX - this won't handle additional build elements correctly
-  my %map = (
-	     lib     => ['lib', 'perl5'],
-	     arch    => ['lib', 'perl5', $self->{config}{archname}],
-	     bin     => ['bin'],
-	     script  => ['bin'],
-	     bindoc  => ['man', 'man1'],
-	     libdoc  => ['man', 'man3'],
-	    );
-  return unless exists $map{$type};
-  return File::Spec->catdir(@{$map{$type}});
+sub install_base_relpaths {
+  # Usage: install_base_relpaths('lib')  or install_base_relpaths();
+  my $self = shift;
+  my $map = $self->{properties}{install_base_relpaths};
+  return $map unless @_;
+  
+  my $type = shift;
+  return unless exists $map->{$type};
+  return File::Spec->catdir(@{$map->{$type}});
 }
 
+
+# Translated from ExtUtils::MM_Any::init_INSTALL_from_PREFIX
+sub prefix_relative {
+  my ($self, $type) = @_;
+  my $installdirs = $self->installdirs;
+
+  return $self->_prefixify($self->install_sets->{$installdirs}{$type},
+			   $self->original_prefix->{$installdirs}, 
+			   $type,
+			  );
+}
+
+
+# Defaults to use in case the config install paths cannot be prefixified.
+sub prefix_relpaths {
+  # Usage: prefix_relpaths('site', 'lib')  or prefix_relpaths('site');
+  my $self = shift;
+  my $installdirs = shift || $self->installdirs;
+  my $map = $self->{properties}{prefix_relpaths}{$installdirs};
+  return $map unless @_;
+  
+  my $type = shift;
+  return unless exists $map->{$type};
+  return File::Spec->catdir(@{$map->{$type}});
+}
+
+
+# Translated from ExtUtils::MM_Unix::prefixify()
+sub _prefixify {
+  my($self, $path, $sprefix, $type) = @_;
+  my $rprefix = $self->prefix;
+  
+  $rprefix .= '/' if $sprefix =~ m|/$|;
+  
+  $self->log_verbose("  prefixify $path from $sprefix to $rprefix\n");
+  
+  if( length $path == 0 ) {
+    $self->log_verbose("  no path to prefixify.\n");
+    # XXX the 'return' here is not a very good failure mechanism
+    return;
+  } elsif( !File::Spec->file_name_is_absolute($path) ) {
+    $self->log_verbose("    path is relative, not prefixifying.\n");
+  } elsif( $sprefix eq $rprefix ) {
+    $self->log_verbose("  no new prefix.\n");
+  } elsif( $path !~ s{^\Q$sprefix\E\b}{}s ) {
+    $self->log_verbose("    cannot prefixify, falling back to default.\n");
+    my $default = $self->prefix_relpaths($self->installdirs, $type);
+    if( !$default ) {
+      $self->log_verbose("    no default install location for type '$type', using prefix '$rprefix'.\n");  
+      return $rprefix;
+    }
+  
+    return $default;
+  }
+  
+  $self->log_verbose("    now $path in $rprefix\n");
+
+  return $path;
+}
+
+
 sub install_destination {
   my ($self, $type) = @_;
   my $p = $self->{properties};
   
   return $p->{install_path}{$type} if exists $p->{install_path}{$type};
-  return File::Spec->catdir($p->{install_base}, $self->install_base_relative($type)) if $p->{install_base};
+  return File::Spec->catdir($p->{install_base}, $self->install_base_relpaths($type)) if $p->{install_base};
+  return File::Spec->catdir($p->{prefix}, $self->prefix_relative($type)) if $p->{prefix};
   return $p->{install_sets}{ $p->{installdirs} }{$type};
 }
 
@@ -2714,12 +3021,19 @@ sub compile_xs {
     my $xsubpp = Module::Build::ModuleInfo->find_module_by_name('ExtUtils::xsubpp')
       or die "Can't find ExtUtils::xsubpp in INC (@INC)";
     
-    my $typemap =  Module::Build::ModuleInfo->find_module_by_name('ExtUtils::typemap', \@INC);
+    my @typemaps;
+    push @typemaps, Module::Build::ModuleInfo->find_module_by_name('ExtUtils::typemap', \@INC);
+    my $lib_typemap = Module::Build::ModuleInfo->find_module_by_name('typemap', ['lib']);
+    if (defined $lib_typemap and -e $lib_typemap) {
+      push @typemaps, 'typemap';
+    }
+    my $typemaps = join ' ', map qq{-typemap "$_"}, @typemaps;
+
     my $cf = $self->{config};
     my $perl = $self->{properties}{perl};
     
     my $command = (qq{$perl "-I$cf->{installarchlib}" "-I$cf->{installprivlib}" "$xsubpp" -noprototypes } .
-		   qq{-typemap "$typemap" "$file"});
+		   qq{$typemaps "$file"});
     
     $self->log_info($command);
     my $fh = IO::File->new("> $args{outfile}") or die "Couldn't write $args{outfile}: $!";
