@@ -74,7 +74,8 @@ sub _init {
     }
   }
 
-  $self->{version} = $self->{versions}{$self->{module}};
+  $self->{version} = $self->{versions}{$self->{module}}
+      if defined( $self->{module} );
 
   return $self;
 }
@@ -104,7 +105,7 @@ sub _parse_file {
   my $fh = IO::File->new( $filename )
     or die( "Can't open '$filename': $!" );
 
-  my( $in_pod, $seen_end ) = ( 0, 0 );
+  my( $in_pod, $seen_end, $need_vers ) = ( 0, 0,0 );
   my( @pkgs, %vers, %pod, @pod );
   my $pkg = 'main';
   my $pod_sect = '';
@@ -140,12 +141,14 @@ sub _parse_file {
 
       if ( $line =~ $PKG_REGEXP ) {
         $pkg = $1;
-        $vers{$pkg} = undef;
-        push( @pkgs, $pkg );
+        push( @pkgs, $pkg ) unless grep( $pkg eq $_, @pkgs );
+        $vers{$pkg} = undef unless exists( $vers{$pkg} );
+	$need_vers = 1;
 
       # first non-comment line in undeclared package main is VERSION
       } elsif ( !exists($vers{main}) && $pkg eq 'main' &&
-		$line =~ $VERS_REGEXP ) { 
+		$line =~ $VERS_REGEXP ) {
+	  $need_vers = 0;
           my $v = $self->_evaluate_version_line( $line );
 	  $vers{$pkg} = $v;
 	  push( @pkgs, 'main' );
@@ -153,15 +156,20 @@ sub _parse_file {
       # first non-comement line in undeclared packge defines package main
       } elsif ( !exists($vers{main}) && $pkg eq 'main' &&
 		$line =~ /\w+/ ) {
+	$need_vers = 1;
 	$vers{main} = '';
 	push( @pkgs, 'main' );
 
-      } elsif ( $line =~ $VERS_REGEXP ) {
+      } elsif ( $line =~ $VERS_REGEXP && $need_vers ) {
         # only first keep if this is the first $VERSION seen
-        unless ( defined $vers{$pkg} && length $vers{$pkg} ) {
-          my $v = $self->_evaluate_version_line( $line );
+	$need_vers = 0;
+        my $v = $self->_evaluate_version_line( $line );
+	unless ( defined $vers{$pkg} && length $vers{$pkg} ) {
 	  $vers{$pkg} = $v;
-        }
+        } else {
+	  warn "Package '$pkg' already declared with version '$vers{$pkg}'\n" .
+	       "  ignoring new version '$v'.\n";
+	}
 
       }
 
@@ -226,9 +234,28 @@ sub packages_inside { @{$_[0]->{packages}}      }
 sub pod_inside      { @{$_[0]->{pod_headings}}  }
 sub contains_pod    { $#{$_[0]->{pod_headings}} }
 
-sub version { $_[0]->{versions}{ $_[1] || $_[0]->{module} } }
+sub version {
+    my $self = shift;
+    my $mod  = shift || $self->{module};
+    my $vers;
+    if ( defined( $mod ) && length( $mod ) &&
+	 exists( $self->{versions}{$mod} ) ) {
+	return $self->{versions}{$mod};
+    } else {
+	return undef;
+    }
+}
 
-sub pod { $_[0]->{pod}{$_[1]} }
+sub pod {
+    my $self = shift;
+    my $sect = shift;
+    if ( defined( $sect ) && length( $sect ) &&
+	 exists( $self->{pod}{$sect} ) ) {
+	return $self->{pod}{$sect};
+    } else {
+	return undef;
+    }
+}
 
 1;
 
