@@ -75,9 +75,9 @@ sub ACTION_install {
   my $self = shift;
   
   return $self->SUPER::ACTION_install(@_)
-    if ExtUtils::Install->VERSION ge '1.30';
+    if eval {ExtUtils::Install->VERSION('1.30'); 1};
     
-  no warnings 'redefine';
+  local $^W = 0; # Avoid a 'redefine' warning
   local *ExtUtils::Install::find = sub {
     my ($code, @dirs) = @_;
 
@@ -89,8 +89,6 @@ sub ACTION_install {
   return $self->SUPER::ACTION_install(@_);
 }
 
-sub need_prelink_c { 1 }
-
 1;
 __END__
 
