@@ -9,28 +9,12 @@ BEGIN {
   push @INC, $t_lib; # Let user's installed version override
 }
 
-use Test();
 
 sub have_module {
   my $module = shift;
   return eval "use $module; 1";
 }
 
-sub need_module {
-  my $module = shift;
-  skip_test("$module not installed") unless have_module($module);
-}
-
-sub skip_test {
-  my $msg = @_ ? shift() : '';
-  print "1..0 # Skipped: $msg\n";
-  exit;
-}
-
-sub skip_subtest {
-  my $msg = @_ ? shift() : '(no reason given)';
-  Test::skip "skip $msg", 1;
-}
 
 sub save_handle {
   my ($handle, $subr) = @_;
