@@ -20,15 +20,20 @@ sub new {
   my %options = @_;
 
   $options{name} ||= 'Simple';
-  $options{dir}  ||= File::Spec->curdir();
+  $options{dir}  ||= Cwd::cwd();
 
   my %data = (
     skip_manifest => 0,
-    xs_module     => 0,
+    xs            => 0,
     %options,
   );
   my $self = bless( \%data, $package );
 
+  if ( -d $self->dirname ) {
+    warn "Warning: Removing existing directory at '@{[$self->dirname]}'\n";
+    $self->remove;
+  }
+
   $self->_gen_default_filedata();
 
   return $self;
@@ -40,8 +45,6 @@ sub _gen_default_filedata {
 
   $self->add_file( 'Build.PL', <<"---" ) unless $self->{filedata}{'Build.PL'};
 use strict;
-use warnings;
-
 use Module::Build;
 
 my \$builder = Module::Build->new(
@@ -55,14 +58,14 @@ my \$builder = Module::Build->new(
   my $module_filename =
     join( '/', ('lib', split(/::/, $self->{name})) ) . '.pm';
 
-  unless ( $self->{xs_module} ) {
+  unless ( $self->{xs} ) {
     $self->add_file( $module_filename, <<"---" ) unless $self->{filedata}{$module_filename};
 package $self->{name};
 
+use vars qw( \$VERSION );
 \$VERSION = '0.01';
 
 use strict;
-use warnings;
 
 1;
 
@@ -70,7 +73,15 @@ __END__
 
 =head1 NAME
 
-$self->{name}
+$self->{name} - Perl extension for blah blah blah
+
+=head1 DESCRIPTION
+
+Stub documentation for $self->{name}.
+
+=head1 AUTHOR
+
+A. U. Thor, a.u.thor\@a.galaxy.far.far.away
 
 =cut
 ---
@@ -89,12 +100,29 @@ package $self->{name};
 
 use base qw( Exporter DynaLoader );
 
-our \$VERSION = '0.01';
-our \@EXPORT_OK = qw( ok );
+use vars qw( \$VERSION \@EXPORT_OK );
+\$VERSION = '0.01';
+\@EXPORT_OK = qw( ok );
 
 bootstrap $self->{name} \$VERSION;
 
 1;
+
+__END__
+
+=head1 NAME
+
+$self->{name} - Perl extension for blah blah blah
+
+=head1 DESCRIPTION
+
+Stub documentation for $self->{name}.
+
+=head1 AUTHOR
+
+A. U. Thor, a.u.thor\@a.galaxy.far.far.away
+
+=cut
 ---
 
     my $xs_filename =
@@ -134,10 +162,14 @@ sub _gen_manifest {
     $self->remove();
     die "Can't write '$manifest'\n";
   };
-  print $fh join( "\n", sort keys( %{$self->{filedata}} ) );
-  print $fh "\n";
+
+  my @files = ( 'MANIFEST', keys %{$self->{filedata}} );
+  my $data = join( "\n", sort @files ) . "\n";
+  print $fh $data;
   close( $fh );
 
+  $self->{filedata}{MANIFEST} = $data;
+  $self->{pending}{change}{MANIFEST} = 1;
 }
 
 sub name { shift()->{name} }
@@ -170,7 +202,9 @@ sub regen {
     foreach my $file ( @files ) {
       my $real_filename = $self->_real_filename( $file );
       my $fullname = File::Spec->catfile( $dist_dirname, $real_filename );
-      unlink( $fullname ) or die "Couldn't unlink '$file'\n";
+      if ( -e $fullname ) {
+	unlink( $fullname ) || die "Couldn't unlink '$file'\n";
+      }
       print "Unlinking pending file '$file'\n" if $VERBOSE;
       delete( $self->{pending}{remove}{$file} );
     }
@@ -222,6 +256,7 @@ sub clean {
 
   my $here  = Cwd::abs_path();
   my $there = File::Spec->rel2abs( $self->dirname() );
+
   if ( -d $there ) {
     chdir( $there ) or die "Can't change directory to '$there'\n";
   } else {
@@ -304,12 +339,13 @@ __END__
 
 =head1 NAME
 
-DistGen
+DistGen - Creates simple distributions for testing.
 
 
 =head1 DESCRIPTION
 
 
+
 =head1 API
 
 
@@ -332,7 +368,7 @@ The name of the module this distribution represents. The default is
 The directory in which to create the distribution directory. The
 default is File::Spec->curdir.
 
-=item xs_module
+=item xs
 
 Generates an XS based module.
 
